@@ -24,9 +24,23 @@ sub length {
 _alias size => 'length';
 
 sub as_string {
-  $self->{document};
+    $self->{document};
 }
 
+sub digest {
+    my $digest;
+    if( Encode::is_utf8 $self->{document} ){
+	Encode::_utf8_off $self->{document};
+	$digest = md5 $self->{document};
+	Encode::_utf8_on $self->{document};
+    }
+    else {
+	$digest = md5 $self->{document};
+    }
+    return $digest;
+}
+
+
 sub is_utf8 {
   Encode::is_utf8($self->{document});
 }
@@ -80,14 +94,18 @@ sub guess_encoding {
 
 use Compress::Zlib;
 chain_sub try_compress {
-  my $td = Compress::Zlib::memGzip $self->{document};
-  $self->{document} = $td if $td;
+    if($self->{document}){
+	my $td = Compress::Zlib::memGzip $self->{document};
+	$self->{document} = $td if $td;
+    }
 }
 
 _alias try_uncompress => 'try_decompress';
 chain_sub try_decompress {
-  my $td = Compress::Zlib::memGunzip $self->{document};
-  $self->{document} = $td if $td;
+    if($self->{document}){
+	my $td = Compress::Zlib::memGunzip $self->{document};
+	$self->{document} = $td if $td;
+    }
 }
 
 ######################################################################
@@ -255,6 +273,9 @@ chain_sub xpath {
     $self->{document} = $t;
 }
 
+chain_sub hpath {
+    $self->html_to_xhtml->xpath(@_);
+}
 
 ######################################################################
 # Tokenization
